/**
 * MK4duo Firmware for 3D Printer, Laser and CNC
 *
 * Based on Marlin, Sprinter and grbl
 * Copyright (C) 2011 Camiel Gubbels / Erik van der Zalm
 * Copyright (C) 2013 Alberto Cotronei @MagoKimbra
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * ----------------------------------------------------------------------------------------
 *
 * MK4duo - Main Include
 * This includes all the other include files in the right order and defines some globals.
 *
 * Alberto Cotronei @MagoKimbra
 * Blog: http://www.marlinkimbra.it
 * WiKI: https://mk4duowiki.altervista.org/wiki/doku.php
 *
 * Licence: GPL
 *
 *******************************************************************************************/

#ifndef _MK4DUO_H_
#define _MK4DUO_H_

#include "Arduino.h"
#include "pins_arduino.h"

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <inttypes.h>

#ifdef __AVR__
  #include <avr/pgmspace.h>
  #include <avr/eeprom.h>
  #include <avr/interrupt.h>
#endif

#include "src/inc/macros.h"
#include "Boards.h"

/**
 * Configuration settings loading
 */

#include "Configuration_Version.h"
#include "Configuration_Basic.h"
  

  #if IS_CARTESIAN
    #include "Configuration_Cartesian.h"
  #endif

  #include "Configuration_Temperature.h"
  #include "Configuration_Feature.h"


#if ENABLED(LASER)
  #include "Configuration_Laser.h"
#endif

#if ENABLED(CNCROUTER)
  #include "Configuration_CNCRouter.h"
#endif

#if ENABLED(HAVE_TMCDRIVER) || ENABLED(HAVE_TMC2130) || ENABLED(HAVE_TMC2208) || ENABLED(HAVE_L6470DRIVER)
  #include "Configuration_Motor_Driver.h"
#endif

/**
 * Modules loading
 */

// Include modules
#include "src/inc/conditionals_pre.h"
#include "src/inc/pins.h"
#include "src/inc/conditionals_post.h"
#include "src/inc/sanitycheck.h"
#include "src/inc/point_t.h"

// HAL Modules
#include "src/HAL/HAL.h"

// Core modules
#include "src/core/mechanics/mechanics.h"
#include "src/core/tools/tools.h"
#include "src/core/tools/nozzle.h"
#include "src/core/fan/fan.h"
#include "src/core/commands/commands.h"
#include "src/core/eeprom/eeprom.h"
#include "src/core/printer/printer.h"
#include "src/core/planner/planner.h"
#include "src/core/endstop/endstops.h"
#include "src/core/stepper/stepper.h"
#include "src/core/heater/sensor/thermistor.h"
#include "src/core/heater/heater.h"
#include "src/core/temperature/temperature.h"

// LCD modules
#include "src/lcd/language/language.h"
#include "src/lcd/ultralcd.h"
#include "src/lcd/nextion/Nextion_lcd.h"

// SD modules
#include "src/sd/cardreader.h"

// Utility modules
#include "src/utility/utility.h"
#include "src/utility/hex_print_routines.h"
#include "src/utility/bezier.h"

// Feature modules
#include "src/feature/printcounter/duration_t.h"
#include "src/feature/printcounter/printcounter.h"
#include "src/feature/probe/probe.h"
#include "src/feature/bedlevel/bedlevel.h"
#include "src/feature/external_dac/external_dac.h"
#include "src/feature/tmc/tmc.h"
#include "src/feature/power/power.h"
#include "src/feature/mixing/mixing.h"
#include "src/feature/laser/base64/base64.h"
#include "src/feature/laser/laser.h"
#include "src/feature/cncrouter/cncrouter.h"

/**
 * External libraries loading
 */

#if HAS_DIGIPOTSS
  #include <SPI.h>
#endif

#if ENABLED(HAVE_TMCDRIVER)
  #include <SPI.h>
  #include <TMC26XStepper.h>
#endif

#if ENABLED(HAVE_TMC2130)
  #include <TMC2130Stepper.h>
#endif

#if ENABLED(HAVE_TMC2208)
  #include <TMC2208Stepper.h>
#endif

#if ENABLED(HAVE_L6470DRIVER)
  #include <SPI.h>
  #include <L6470.h>
#endif


#endif /* _MK4DUO_H_ */
