/**
 * MK4duo Firmware for 3D Printer, Laser and CNC
 *
 * Based on Marlin, Sprinter and grbl
 * Copyright (C) 2011 Camiel Gubbels / Erik van der Zalm
 * Copyright (C) 2013 Alberto Cotronei @MagoKimbra
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 */

/**
 * Nextion_lcd.cpp
 *
 * Copyright (c) 2014-2016 Alberto Cotronei @MagoKimbra
 *
 * Grbl is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Grbl is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Grbl. If not, see <http://www.gnu.org/licenses/>.
 */

#include "../../../MK4duo.h"

#if ENABLED(NEXTION)

  #include "Nextion_lcd.h"
  #include "library/Nextion.h"

  bool        NextionON                 = false,
              show_Wave                 = true,
              lcdDrawUpdate             = false,
              lcd_clicked               = false;
  uint8_t     PageID                    = 0,
              lcd_status_message_level  = 0;
  uint16_t    slidermaxval              = 20;
  char        buffer[70]                = { 0 };
  char        lcd_status_message[30]    = WELCOME_MSG;
  const float manual_feedrate_mm_m[]    = MANUAL_FEEDRATE;

  #if HAS_SDSUPPORT
    // 0 card not present, 1 SD not insert, 2 SD insert, 3 SD printing
    enum SDstatus_enum {NO_SD = 0, SD_NO_INSERT = 1, SD_INSERT = 2, SD_PRINTING = 3, SD_PAUSE = 4 };
    SDstatus_enum SDstatus    = NO_SD;
    NexUpload Firmware(NEXTION_FIRMWARE_FILE, 57600);
  #endif

  #if ENABLED(NEXTION_GFX)
    GFX gfx = GFX(1, 1, 1, 1);
  #endif

  /**
   *******************************************************************
   * Nextion component all page
   *******************************************************************
   */
  NexObject Pstart        = NexObject(0,  0,  "start");
  NexObject Pprinter      = NexObject(1,  0,  "printer");
  NexObject Psdcard       = NexObject(2,  0,  "sdcard");
  NexObject Pmove         = NexObject(3,  0,  "move");
  NexObject Pspeed        = NexObject(4,  0,  "speed");
  NexObject Pbrightness   = NexObject(5,  0,  "brightness");
 

  /**
   *******************************************************************
   * Nextion component for page:start
   *******************************************************************
   */
  NexObject startimer     = NexObject(0,  1,  "tm0");

  
  /**
   *******************************************************************
   * Nextion component for page:printer
   *******************************************************************
   */
  
  NexObject SD          = NexObject(1,  4,  "sd");
  NexObject VSpeed      = NexObject(1,  5,  "vs");
  NexObject LcdStatus   = NexObject(1,  2,  "t0");
  NexObject progressbar = NexObject(1,  1,  "j0");
  NexObject XStatus     = NexObject(1, 12,  "t1");
  NexObject YStatus     = NexObject(1, 13,  "t2");
  NexObject ZStatus     = NexObject(1, 14,  "t3");  
  NexObject Stop        = NexObject(1,  8,  "m7");
  NexObject PlayPause   = NexObject(1, 16,  "p0");
  
  /**
   *******************************************************************
   * Nextion component for page:SDCard
   *******************************************************************
   */
  NexObject sdlist      = NexObject(2,   1, "h0");
  NexObject sdrow0      = NexObject(2,   2, "t0");
  NexObject sdrow1      = NexObject(2,   3, "t1");
  NexObject sdrow2      = NexObject(2,   4, "t2");
  NexObject sdrow3      = NexObject(2,   5, "t3");
  NexObject sdrow4      = NexObject(2,   6, "t4");
  NexObject sdrow5      = NexObject(2,   7, "t5");
  NexObject Folder0     = NexObject(2,   8, "p0");
  NexObject Folder1     = NexObject(2,   9, "p1");
  NexObject Folder2     = NexObject(2,  10, "p2");
  NexObject Folder3     = NexObject(2,  11, "p3");
  NexObject Folder4     = NexObject(2,  12, "p4");
  NexObject Folder5     = NexObject(2,  13, "p5"); 
  NexObject Folderup    = NexObject(2,  14, "p6");
  NexObject sdfolder    = NexObject(2,  16, "t6");
  NexObject sd_mount    = NexObject(2,  21, "p12");
  NexObject sd_dismount = NexObject(2,  22, "p13");

  /**
   *******************************************************************
   * Nextion component for page:Move
   *******************************************************************
   */
  NexObject XYZero      = NexObject(3,   2, "p4");
  NexObject XYUp        = NexObject(3,   3, "p5");
  NexObject XYRight     = NexObject(3,   4, "p6");
  NexObject XYDown      = NexObject(3,   5, "p7");
  NexObject XYLeft      = NexObject(3,   6, "p8");
  NexObject ZZero       = NexObject(3,   7, "p9");
  NexObject ZUp         = NexObject(3,   8, "p10");
  NexObject ZDown       = NexObject(3,   9, "p11");
  NexObject movecmd     = NexObject(3,  11, "vacmd");
  NexObject LedCoord5   = NexObject(3,  12, "t0");
  NexObject ext         = NexObject(3,  17, "va0");
  NexObject SpeedX      = NexObject(3,  18, "vafrx");
  NexObject SpeedY      = NexObject(3,  19, "vafry");
  NexObject SpeedZ      = NexObject(3,  20, "vafrz");
  NexObject XYZHome     = NexObject(3,  23, "m0");
  NexObject XZero       = NexObject(3,  24, "m1");
  NexObject YZero       = NexObject(3,  25, "m2");
  NexObject XYZZero     = NexObject(3,  26, "m3");

  /**
   *******************************************************************
   * Nextion component for page:Speed
   *******************************************************************
   */
  NexObject Speed       = NexObject(4,  4,  "h0");

  /**
   *******************************************************************
   * Nextion component for page:Brightness
   *******************************************************************
   */



  NexObject *nex_listen_list[] =
  {
    // Page printer touch listen
    &Stop, &PlayPause,

    // Page sdcard touch listen
    &sdlist, &sdrow0, &sdrow1, &sdrow2,
    &sdrow3, &sdrow4, &sdrow5, &Folderup, &sd_mount, &sd_dismount,


    // Page move touch listen
    &XYZero, &XYUp, &XYRight, &XYDown, &XYLeft,
    &ZZero, &ZUp, &ZDown, &XYZHome, &XZero, &YZero, &XYZZero,

    // Page speed touch listen
    &Speed,
    NULL
  };

  NexObject *row_list[] =
  {
    &sdrow0,
    &sdrow1,
    &sdrow2,
    &sdrow3,
    &sdrow4,
    &sdrow5,
    NULL
  };

  NexObject *folder_list[] =
  {
    &Folder0,
    &Folder1,
    &Folder2,
    &Folder3,
    &Folder4,
    &Folder5,
    NULL
  };

  NexObject *speed_list[] =
  {
    &SpeedX,
    &SpeedY,
    &SpeedZ,
    NULL
  };

  // Function pointer to menu functions.
  typedef void (*screenFunc_t)();

  /**
   *
   * Menu actions
   *
   */
  void menu_action_back() { Pprinter.show(); }
  void menu_action_function(screenFunc_t func) { (*func)(); }

  void setpagePrinter() {
    char temp[10] = { 0 };

   

    LOOP_XYZE(i) {
      ZERO(temp);
      itoa(manual_feedrate_mm_m[i], temp, 10);
      speed_list[i]->setText(temp, "move");
    }

    #if HAS_SDSUPPORT
      card.mount();
      if (card.cardOK)
        SDstatus = SD_INSERT;
      else
        SDstatus = SD_NO_INSERT;
      SD.setValue(SDstatus, "printer");
    #endif

    VSpeed.setValue(100, "printer");

  
  }


  /**
   * START_SCREEN  Opening code for a screen having only static items.
   *               Do simplified scrolling of the entire screen.
   *
   * START_MENU    Opening code for a screen with menu items.
   *               Scroll as-needed to keep the selected line in view.
   */
  #define START_SCREEN() \
    start_menu(false, true); \
    do { \
      uint8_t _lcdLineNr = 0; \

  #define START_MENU() \
    start_menu(true, true); \
    uint32_t encoderLine; \
    uint8_t _lcdLineNr = 0; \
    do { \
      _lcdLineNr = 0; \
      LcdPos.getValue(&encoderLine); \
      HAL::delayMilliseconds(200)

  #define MENU_ITEM(TYPE, LABEL, ...) \
      if (lcdDrawUpdate) { \
        lcd_row_list[_lcdLineNr]->setText(PSTR(LABEL)); \
        LcdMax.setValue(_lcdLineNr); \
      } \
      if (lcd_clicked && encoderLine == _lcdLineNr) { \
        menu_action_ ## TYPE(__VA_ARGS__); \
        return; \
      } \
      ++_lcdLineNr

  #define MENU_BACK(LABEL) MENU_ITEM(back, LABEL)

  #define STATIC_ITEM(LABEL) \
      if (lcdDrawUpdate) { \
        lcd_row_list[_lcdLineNr]->setText(PSTR(LABEL)); \
        LcdMin.setValue(_lcdLineNr + 1); \
      } \
      ++_lcdLineNr \

  #define END_MENU() \
      printer.idle(); \
      lcdDrawUpdate = false; \
    } while(1)

  #define END_SCREEN() \
      lcdDrawUpdate = false; \
    } while(0)

  #if HAS_SDSUPPORT

    void UploadNewFirmware() {
      if (IS_SD_INSERTED || card.cardOK) {
        Firmware.startUpload();
        nexSerial.end();
        lcd_init();
      }
    }

    void printrowsd(uint8_t row, const bool folder, const char* filename) {
      if (folder) {
        folder_list[row]->SetVisibility(true);
        row_list[row]->attachPop(sdfolderPopCallback, row_list[row]);
      } else if (filename == "") {
        folder_list[row]->SetVisibility(false);
        row_list[row]->detachPop();
      } else {
        folder_list[row]->SetVisibility(false);
        row_list[row]->attachPop(sdfilePopCallback, row_list[row]);
      }
      row_list[row]->setText(filename);
    }

    static void setrowsdcard(uint32_t number = 0) {
      uint16_t fileCnt = card.get_num_Files();
      uint32_t i = 0;
      card.getWorkDirName();

      if (card.fileName[0] != '/') {
        Folderup.SetVisibility(true);
        Folderup.attachPop(sdfolderUpPopCallback);
        sdfolder.setText(card.fileName);
      } else {
        Folderup.detachPop();
        Folderup.SetVisibility(false);
        sdfolder.setText("");
      }

      if (fileCnt) {
        for (uint8_t row = 0; row < 6; row++) {
          i = row + number;
          if (i < fileCnt) {
            #if ENABLED(SDCARD_SORT_ALPHA)
              card.getfilename_sorted(i);
            #else
              card.getfilename(i);
            #endif
            printrowsd(row, card.filenameIsDir, card.fileName);
          } else {
            printrowsd(row, false, "");
          }
        }
      }
      sendCommand("ref 0");
    }

    static void menu_action_sdfile(const char* filename) {
      card.openAndPrintFile(filename);
      Pprinter.show();
    }

    static void menu_action_sddirectory(const char* filename) {
      card.chdir(filename);
      setpageSD();
    }

    void setpageSD() {
      uint16_t fileCnt = card.get_num_Files();

      if (fileCnt <= 6)
        slidermaxval = 0;
      else
        slidermaxval  = fileCnt - 6;

      uint16_t hig = 210 - slidermaxval * 10;
      if (hig < 10) hig = 10;

      sdlist.Set_cursor_height_hig(hig);
      sdlist.setMaxval(slidermaxval);
      sdlist.setValue(slidermaxval);
      sendCommand("ref 0");

      setrowsdcard();
    }

    void sdmountdismountPopCallback(void *ptr) {
      if (ptr == &sd_mount) {
        card.mount();
        if (card.cardOK)
          SDstatus = SD_INSERT;
        else
          SDstatus = SD_NO_INSERT;
        SD.setValue(SDstatus, "printer");
      }
      else {
        card.unmount();
        SDstatus = SD_NO_INSERT;
        SD.setValue(SDstatus, "printer");
      }
      setpageSD();
    }

    void sdlistPopCallback(void *ptr) {
      UNUSED(ptr);
      uint32_t number = 0;
      sdlist.getValue(&number);
      number = slidermaxval - number;
      setrowsdcard(number);
    }

    void sdfilePopCallback(void *ptr) {
      ZERO(buffer);

      if (ptr == &sdrow0)
        sdrow0.getText(buffer, sizeof(buffer));
      else if (ptr == &sdrow1)
        sdrow1.getText(buffer, sizeof(buffer));
      else if (ptr == &sdrow2)
        sdrow2.getText(buffer, sizeof(buffer));
      else if (ptr == &sdrow3)
        sdrow3.getText(buffer, sizeof(buffer));
      else if (ptr == &sdrow4)
        sdrow4.getText(buffer, sizeof(buffer));
      else if (ptr == &sdrow5)
        sdrow5.getText(buffer, sizeof(buffer));

      menu_action_sdfile(buffer);
    }

    void sdfolderPopCallback(void *ptr) {
      ZERO(buffer);

      if (ptr == &sdrow0)
        sdrow0.getText(buffer, sizeof(buffer));
      else if (ptr == &sdrow1)
        sdrow1.getText(buffer, sizeof(buffer));
      else if (ptr == &sdrow2)
        sdrow2.getText(buffer, sizeof(buffer));
      else if (ptr == &sdrow3)
        sdrow3.getText(buffer, sizeof(buffer));
      else if (ptr == &sdrow4)
        sdrow4.getText(buffer, sizeof(buffer));
      else if (ptr == &sdrow5)
        sdrow5.getText(buffer, sizeof(buffer));

      menu_action_sddirectory(buffer);
    }

    void sdfolderUpPopCallback(void *ptr) {
      UNUSED(ptr);
      card.updir();
      setpageSD();
    }

    void setplaypausePopCallback(void *ptr) {
      UNUSED(ptr);

      if (card.cardOK && card.isFileOpen()) {
        if (IS_SD_PRINTING) {
          card.pauseSDPrint();
          print_job_counter.pause();
        }
        else {
          card.startFileprint();
          print_job_counter.start();
        }
      }
    }

  #endif


  void setstopPopCallback(void *ptr) {
    UNUSED(ptr);
	
    ZERO(buffer);
    commands.enqueue_and_echo_P(PSTR("M112"));
  }



  void setmovePopCallback(void *ptr) {
    UNUSED(ptr);

    ZERO(buffer);
    movecmd.getText(buffer, sizeof(buffer));
    commands.enqueue_and_echo_P(PSTR("G91"));
    commands.enqueue_and_echo(buffer);
    commands.enqueue_and_echo_P(PSTR("G90"));

 
  }

  void motoroffPopCallback(void *ptr) {
    UNUSED(ptr);
    commands.enqueue_and_echo_P(PSTR("M84"));
  }

  void sendPopCallback(void *ptr) {
    UNUSED(ptr);
    lcd_clicked = true;
    printer.setWaitForUser(false);
  }

 

  void lcd_init() {
    HAL::delayMilliseconds(2000);

    for (uint8_t i = 0; i < 10; i++) {
      ZERO(buffer);
      NextionON = nexInit(buffer);
      if (NextionON) break;
      HAL::delayMilliseconds(1000);
    }

    if (!NextionON) {
      SERIAL_LM(ER, "Nextion not connected!");
      return;
    }
    else {
      SERIAL_MSG("Nextion");
	  
      // Get Model

      if (strstr(buffer, "3224")) {       // Model 2.4" or 2.8" Normal or Enhanced
        SERIAL_MSG(" 2.4");
      }
      else if (strstr(buffer, "4827")) {  // Model 4.3" Normal or Enhanced
        SERIAL_MSG(" 4.3");
      }
      else if (strstr(buffer, "8048")) {  // Model 7" Normal or Enhanced
        SERIAL_MSG(" 7");
      }
      SERIAL_CHR('"'); SERIAL_EM(" connected!");

      #if HAS_SDSUPPORT
        sd_mount.attachPop(sdmountdismountPopCallback, &sd_mount);
        sd_dismount.attachPop(sdmountdismountPopCallback, &sd_dismount);
        sdlist.attachPop(sdlistPopCallback);
      #endif

     
      XYZHome.attachPop(setmovePopCallback);
      XYZero.attachPop(setmovePopCallback);
      XZero.attachPop(setmovePopCallback);	  
      YZero.attachPop(setmovePopCallback);	  
      XYZZero.attachPop(setmovePopCallback);	  
      XYUp.attachPop(setmovePopCallback);
      XYRight.attachPop(setmovePopCallback);
      XYDown.attachPop(setmovePopCallback);
      XYLeft.attachPop(setmovePopCallback);
      ZZero.attachPop(setmovePopCallback);
      ZUp.attachPop(setmovePopCallback);
      ZDown.attachPop(setmovePopCallback);
      Stop.attachPop(setstopPopCallback);	  
      PlayPause.attachPop(setplaypausePopCallback);

      setpagePrinter();
      startimer.enable();
    }
  }





  static void coordtoLCD() {
    char* valuetemp;
    ZERO(buffer);

    if (PageID == 1) {
    
	    valuetemp = ftostr52sign(LOGICAL_X_POSITION(mechanics.current_position[X_AXIS]));
        strcat(buffer, valuetemp);
        XStatus.setText(buffer);
		ZERO(buffer);
    
        valuetemp = ftostr52sign(LOGICAL_Y_POSITION(mechanics.current_position[Y_AXIS]));
        strcat(buffer, valuetemp);
        YStatus.setText(buffer);
		ZERO(buffer);

    
        valuetemp = ftostr52sign(LOGICAL_Z_POSITION(mechanics.current_position[Z_AXIS]));
        strcat(buffer, valuetemp);
    	ZStatus.setText(buffer);
    }
    else if (PageID == 3) {

        valuetemp = ftostr52sign(LOGICAL_X_POSITION(mechanics.current_position[X_AXIS]));
        strcat(buffer, "X");
        strcat(buffer, valuetemp);
      
    
        valuetemp = ftostr52sign(LOGICAL_Y_POSITION(mechanics.current_position[Y_AXIS]));
        strcat(buffer, " Y");
        strcat(buffer, valuetemp);
    
    

    
        valuetemp = ftostr52sign(LOGICAL_Z_POSITION(mechanics.current_position[Z_AXIS]));
        strcat(buffer, " Z");
        strcat(buffer, valuetemp);
      
      

      LedCoord5.setText(buffer);
    }
   }

  void lcd_key_touch_update() {
    if (!NextionON) return;
    nexLoop(nex_listen_list);
  }

  void nextion_draw_update() {

    static uint8_t  PreviousPage = 0,
                    Previousfeedrate = 0,
                    PreviousfanSpeed = 0,
                    PreviouspercentDone = 0;
    

    if (!NextionON) return;

    PageID = Nextion_PageID();

    switch (PageID) {
      case 1:
        if (PreviousPage != 1) {
          lcd_setstatus(lcd_status_message);
          }

        if (Previousfeedrate != mechanics.feedrate_percentage) {
          VSpeed.setValue(mechanics.feedrate_percentage);
          Previousfeedrate = mechanics.feedrate_percentage;
        }
 
        coordtoLCD();

        if (PreviouspercentDone != printer.progress) {
          // Progress bar solid part
          progressbar.setValue(printer.progress);
          }

        #if HAS_SDSUPPORT

          if (card.isFileOpen()) {
            if (IS_SD_PRINTING && SDstatus != SD_PRINTING) {
              SDstatus = SD_PRINTING;
              SD.setValue(SDstatus);
            }
            else if (!IS_SD_PRINTING && SDstatus != SD_PAUSE) {
              SDstatus = SD_PAUSE;
              SD.setValue(SDstatus);
            }
          }
          else if (card.cardOK && SDstatus != SD_INSERT) {
            SDstatus = SD_INSERT;
            SD.setValue(SDstatus);
          }
          else if (!card.cardOK && SDstatus != SD_NO_INSERT) {
            SDstatus = SD_NO_INSERT;
            SD.setValue(SDstatus);
          }

        #endif // SDSUPPORT

        break;
      #if HAS_SDSUPPORT
        case 2:
          if (PreviousPage != 2) setpageSD();
          break;
      #endif
      case 3:
        coordtoLCD();
        break;
      case 4:
        static uint32_t temp_feedrate = 0;
        VSpeed.getValue(&temp_feedrate, "printer");
        Previousfeedrate = mechanics.feedrate_percentage = (int)temp_feedrate;
        break;
    }

    PreviousPage = PageID;
  }

  void lcd_setstatus(const char* message, bool persist) {
    UNUSED(persist);
    if (lcd_status_message_level > 0 || !NextionON) return;
    strncpy(lcd_status_message, message, 30);
    if (PageID == 1) LcdStatus.setText(lcd_status_message);
  }

  void lcd_setstatusPGM(const char* message, int8_t level) {
    if (level < 0) level = lcd_status_message_level = 0;
    if (level < lcd_status_message_level || !NextionON) return;
    strncpy_P(lcd_status_message, message, 30);
    lcd_status_message_level = level;
    if (PageID == 1) LcdStatus.setText(lcd_status_message);
  }

  void lcd_status_printf_P(const uint8_t level, const char * const fmt, ...) {
    if (level < lcd_status_message_level || !NextionON) return;
    lcd_status_message_level = level;
    va_list args;
    va_start(args, fmt);
    vsnprintf(lcd_status_message, 30, fmt, args);
    va_end(args);
    if (PageID == 1) LcdStatus.setText(lcd_status_message);
  }

  void lcd_setalertstatusPGM(const char * const message) {
    lcd_setstatusPGM(message, 1);
  }

  void lcd_reset_alert_level() { lcd_status_message_level = 0; }

#endif
